import { useLogger, logger as logger$1, extendViteConfig, defineNuxtModule } from '@nuxt/kit';
import { colors } from 'consola/utils';
import { resolve } from 'node:path';
import { existsSync } from 'node:fs';
import { startSubprocess } from '@nuxt/devtools-kit';
import { getPort } from 'get-port-please';
import { withTrailingSlash } from 'ufo';

const logger = useLogger("nuxt:storybook");

async function setupStorybook(options, nuxt) {
  const STORYBOOK_ROUTE = options.route;
  const STORYBOOK_PORT = await getPort({
    ports: [options.port || 6006, 6007, 6008, 6009, 6010]
  });
  const STORYBOOK_HOST = options.host;
  const STORYBOOK_URL = STORYBOOK_HOST + (STORYBOOK_PORT == 80 ? "" : `:${STORYBOOK_PORT}`);
  const projectDir = resolve(nuxt.options.rootDir);
  const args = isStorybookConfigured(projectDir) ? ["storybook", "dev", "--port", `${STORYBOOK_PORT}`, "--ci"] : [
    "storybook-nuxt",
    "init",
    "--start",
    "--port",
    `${STORYBOOK_PORT}`,
    "--ci"
  ];
  logger$1.verbose(" ");
  logger$1.verbose(
    isStorybookConfigured(projectDir) ? "\u{1F4DA}  Storybook is configured" : "\u{1F4DA}  Storybook is not installed"
  );
  logger$1.verbose("");
  if (!nuxt.options.dev)
    return;
  nuxt.hook("app:resolve", async () => {
    const _process = startSubprocess(
      {
        command: "npx",
        args,
        cwd: projectDir
      },
      {
        id: "nuxt-storybook-module:client",
        name: "Storybook Server Terminal"
      },
      nuxt
    );
    _process.getProcess().stdout?.pipe(process.stdout);
    _process.getProcess().stderr?.pipe(process.stderr);
    nuxt.hook("close", () => {
      logger$1.verbose(" \u26A0\uFE0F Closing Storybook  ");
      return _process.terminate();
    });
    await new Promise((resolve2) => setTimeout(resolve2, 2e3));
    logger$1.verbose("\u2139\uFE0F Storybook ready  ");
  });
  const storybookProxy = {
    target: STORYBOOK_URL,
    changeOrigin: true,
    followRedirects: true,
    secure: false,
    rewrite: (path) => path.replace(STORYBOOK_ROUTE, "")
  };
  extendViteConfig((config) => {
    config.optimizeDeps ??= {};
    config.optimizeDeps.include = config.optimizeDeps.include || [];
    config.server ??= {};
    config.server.proxy ??= {};
    config.server.proxy[STORYBOOK_ROUTE] = storybookProxy;
    config.server.proxy["/@vite/client"] = storybookProxy;
    config.server.proxy["/virtual:/@storybook"] = storybookProxy;
    config.server.proxy["/node_modules"] = storybookProxy;
    config.server.proxy["/.storybook"] = storybookProxy;
    config.server.proxy["/stores"] = storybookProxy;
    config.server.proxy["/stories"] = storybookProxy;
    config.server.proxy["/assets"] = storybookProxy;
    config.server.proxy["/@id"] = storybookProxy;
    config.server.proxy["/@fs"] = storybookProxy;
    config.server.proxy["/app.vue"] = storybookProxy;
    config.server.proxy["/.nuxt"] = storybookProxy;
    config.server.proxy["/app.config.mjs"] = storybookProxy;
    config.server.proxy["/i18n.options.mjs"] = storybookProxy;
    config.server.proxy["/i18n.config.ts"] = storybookProxy;
    config.server.proxy["/components"] = storybookProxy;
    config.server.proxy["/composables"] = storybookProxy;
    config.server.proxy["/layouts"] = storybookProxy;
    config.server.proxy["/pages"] = storybookProxy;
    config.server.proxy["/storybook-server-channel"] = storybookProxy;
  });
  nuxt.hook("build:done", () => {
    logger$1.verbose(" ");
    logger$1.verbose("\u2714 Storybook build done  ");
    logger$1.verbose("  ");
    import.meta.env = import.meta.env || {};
    import.meta.env.__STORYBOOK__ = JSON.stringify(options);
  });
  logger$1.log(
    `  \u279C Storybook: ${colors.underline(withTrailingSlash(STORYBOOK_URL))}`
  );
  nuxt.hook("devtools:customTabs", (tabs) => {
    tabs.push({
      // unique identifier
      name: "nuxt-storybook",
      // title to display in the tab
      title: "Storybook",
      // any icon from Iconify, or a URL to an image
      icon: "devicon:storybook",
      // iframe view
      view: {
        type: "iframe",
        // absolute URL to the iframes
        src: `${STORYBOOK_ROUTE}/`
      }
    });
  });
}
function isStorybookConfigured(rootDir) {
  const isTypeScriptProject = existsSync(resolve(rootDir, "tsconfig.json"));
  const configFileExtension = isTypeScriptProject ? "ts" : "js";
  const sbMain = existsSync(
    resolve(rootDir, `.storybook/main.${configFileExtension}`)
  );
  return sbMain;
}

const module = defineNuxtModule({
  meta: {
    name: "@nuxtjs/storybook",
    configKey: "storybook",
    compatibility: {
      nuxt: "^3.0.0"
    }
  },
  defaults: (nuxt) => ({
    host: import.meta.env?.STORYBOOK_HOST || "http://localhost",
    route: "/_storybook",
    port: 6006,
    logLevel: nuxt.options.logLevel === "silent" ? 0 : 3,
    enabled: true
  }),
  async setup(options, nuxt) {
    if (import.meta.env?.__STORYBOOK__ || !options.enabled)
      return;
    logger.level = options.logLevel;
    logger.verbose("\u{1F50C}  Storybook Module Setup");
    setupStorybook(options, nuxt);
  }
});

export { module as default };
